@@ -22,21 +22,17 @@ import java.nio.Buffer;
 import java.nio.ByteBuffer;
 import java.nio.CharBuffer;
 import java.nio.charset.Charset;
-import java.security.AccessController;
-import java.security.PrivilegedActionException;
-import java.security.PrivilegedExceptionAction;
 import java.util.HashMap;
 import java.util.Map;
 
 import jakarta.servlet.WriteListener;
 import jakarta.servlet.http.HttpServletResponse;
 
-import org.apache.catalina.Globals;
 import org.apache.coyote.ActionCode;
 import org.apache.coyote.CloseNowException;
 import org.apache.coyote.Response;
-import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.C2BConverter;
+import org.apache.tomcat.util.buf.CharsetHolder;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
@@ -116,7 +112,7 @@ public class OutputBuffer extends Writer {
     /**
      * Associated Coyote response.
      */
-    private Response coyoteResponse;
+    private final Response coyoteResponse;
 
 
     /**
@@ -130,29 +126,21 @@ public class OutputBuffer extends Writer {
     /**
      * Create the buffer with the specified initial size.
      *
-     * @param size Buffer size to use
+     * @param size           Buffer size to use
+     * @param coyoteResponse The associated Coyote response
      */
-    public OutputBuffer(int size) {
+    public OutputBuffer(int size, Response coyoteResponse) {
         defaultBufferSize = size;
         bb = ByteBuffer.allocate(size);
         clear(bb);
         cb = CharBuffer.allocate(size);
         clear(cb);
+        this.coyoteResponse = coyoteResponse;
     }
 
 
     // ------------------------------------------------------------- Properties
 
-    /**
-     * Associated Coyote response.
-     *
-     * @param coyoteResponse Associated Coyote response
-     */
-    public void setResponse(Response coyoteResponse) {
-        this.coyoteResponse = coyoteResponse;
-    }
-
-
     /**
      * Is the response output suspended ?
      *
@@ -232,18 +220,17 @@ public class OutputBuffer extends Writer {
             flushCharBuffer();
         }
 
-        // Content length can be calculated if:
-        // - the response has not been committed
-        // AND
-        // - the content length has not been explicitly set
-        // AND
-        // - some content has been written OR this is NOT a HEAD request
-        if ((!coyoteResponse.isCommitted()) && (coyoteResponse.getContentLengthLong() == -1) &&
-                ((bb.remaining() > 0 || !coyoteResponse.getRequest().method().equals("HEAD")))) {
+        if ((!coyoteResponse.isCommitted()) && (coyoteResponse.getContentLengthLong() == -1)) {
+            // If this didn't cause a commit of the response, the final content
+            // length can be calculated.
             coyoteResponse.setContentLength(bb.remaining());
         }
 
-        doFlush(coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS);
+        if (coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS) {
+            doFlush(true);
+        } else {
+            doFlush(false);
+        }
         closed = true;
 
         // The request should have been completely read by the time the response
@@ -329,24 +316,24 @@ public class OutputBuffer extends Writer {
             try {
                 coyoteResponse.doWrite(buf);
             } catch (CloseNowException e) {
-                // Catch this subclass as it requires specific handling.
+                // Catch this sub-class as it requires specific handling.
                 // Examples where this exception is thrown:
                 // - HTTP/2 stream timeout
                 // Prevent further output for this response
                 closed = true;
                 throw e;
-            } catch (IOException ioe) {
+            } catch (IOException e) {
                 // An IOException on a write is almost always due to
                 // the remote client aborting the request. Wrap this
                 // so that it can be handled better by the error dispatcher.
-                throw new ClientAbortException(ioe);
+                throw new ClientAbortException(e);
             }
         }
 
     }
 
 
-    public void write(byte[] b, int off, int len) throws IOException {
+    public void write(byte b[], int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -368,14 +355,14 @@ public class OutputBuffer extends Writer {
     }
 
 
-    private void writeBytes(byte[] b, int off, int len) throws IOException {
+    private void writeBytes(byte b[], int off, int len) throws IOException {
 
         if (closed) {
-            return;
+            throw new IOException(sm.getString("outputBuffer.closed"));
         }
 
         append(b, off, len);
-        bytesWritten += len;
+        updateBytesWritten(len);
 
         // if called from within flush(), then immediately flush
         // remaining bytes
@@ -389,12 +376,12 @@ public class OutputBuffer extends Writer {
     private void writeBytes(ByteBuffer from) throws IOException {
 
         if (closed) {
-            return;
+            throw new IOException(sm.getString("outputBuffer.closed"));
         }
 
         int remaining = from.remaining();
         append(from);
-        bytesWritten += remaining;
+        updateBytesWritten(remaining);
 
         // if called from within flush(), then immediately flush
         // remaining bytes
@@ -407,6 +394,10 @@ public class OutputBuffer extends Writer {
 
     public void writeByte(int b) throws IOException {
 
+        if (closed) {
+            throw new IOException(sm.getString("outputBuffer.closed"));
+        }
+
         if (suspended) {
             return;
         }
@@ -416,8 +407,24 @@ public class OutputBuffer extends Writer {
         }
 
         transfer((byte) b, bb);
-        bytesWritten++;
+        updateBytesWritten(1);
+    }
 
+
+    private void updateBytesWritten(int increment) throws IOException {
+        bytesWritten += increment;
+        int contentLength = coyoteResponse.getContentLength();
+
+        /*
+         * Handle the requirements of section 5.7 of the Servlet specification - Closure of the Response Object.
+         *
+         * Currently this just handles the simple case. There is work in progress to better define what should happen if
+         * an attempt is made to write > content-length bytes. When that work is complete, this is likely where the
+         * implementation will end up.
+         */
+        if (contentLength != -1 && bytesWritten >= contentLength) {
+            close();
+        }
     }
 
 
@@ -441,12 +448,12 @@ public class OutputBuffer extends Writer {
             }
             if (from.remaining() > 0) {
                 flushByteBuffer();
-            } else if (conv.isUnderflow() && bb.limit() > bb.capacity() - 4) {
+            } else if (conv.isUndeflow() && bb.limit() > bb.capacity() - 4) {
                 // Handle an edge case. There are no more chars to write at the
                 // moment but there is a leftover character in the converter
                 // which must be part of a surrogate pair. The byte buffer does
                 // not have enough space left to output the bytes for this pair
-                // once it is complete (it will require 4 bytes) so flush now to
+                // once it is complete )it will require 4 bytes) so flush now to
                 // prevent the bytes for the leftover char and the rest of the
                 // surrogate pair yet to be written from being lost.
                 // See TestOutputBuffer#testUtf8SurrogateBody()
@@ -474,7 +481,7 @@ public class OutputBuffer extends Writer {
 
 
     @Override
-    public void write(char[] c) throws IOException {
+    public void write(char c[]) throws IOException {
 
         if (suspended) {
             return;
@@ -486,7 +493,7 @@ public class OutputBuffer extends Writer {
 
 
     @Override
-    public void write(char[] c, int off, int len) throws IOException {
+    public void write(char c[], int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -545,44 +552,25 @@ public class OutputBuffer extends Writer {
             return;
         }
 
-        Charset charset = coyoteResponse.getCharset();
+        CharsetHolder charsetHolder = coyoteResponse.getCharsetHolder();
+        // setCharacterEncoding() was called with an invalid character set
+        // Trigger an UnsupportedEncodingException
+        charsetHolder.validate();
+        Charset charset = charsetHolder.getCharset();
 
         if (charset == null) {
-            if (coyoteResponse.getCharacterEncoding() != null) {
-                // setCharacterEncoding() was called with an invalid character set
-                // Trigger an UnsupportedEncodingException
-                charset = B2CConverter.getCharset(coyoteResponse.getCharacterEncoding());
-            }
             charset = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET;
         }
 
         conv = encoders.get(charset);
 
         if (conv == null) {
-            conv = createConverter(charset);
+            conv = new C2BConverter(charset);
             encoders.put(charset, conv);
         }
     }
 
 
-    private static C2BConverter createConverter(final Charset charset) throws IOException {
-        if (Globals.IS_SECURITY_ENABLED) {
-            try {
-                return AccessController.doPrivileged(new PrivilegedCreateConverter(charset));
-            } catch (PrivilegedActionException ex) {
-                Exception e = ex.getException();
-                if (e instanceof IOException) {
-                    throw (IOException) e;
-                } else {
-                    throw new IOException(ex);
-                }
-            }
-        } else {
-            return new C2BConverter(charset);
-        }
-    }
-
-
     // -------------------- BufferedOutputStream compatibility
 
     public long getContentWritten() {
@@ -663,7 +651,7 @@ public class OutputBuffer extends Writer {
      *
      * @throws IOException Writing overflow data to the output channel failed
      */
-    public void append(byte[] src, int off, int len) throws IOException {
+    public void append(byte src[], int off, int len) throws IOException {
         if (bb.remaining() == 0) {
             appendByteArray(src, off, len);
         } else {
@@ -686,7 +674,7 @@ public class OutputBuffer extends Writer {
      *
      * @throws IOException Writing overflow data to the output channel failed
      */
-    public void append(char[] src, int off, int len) throws IOException {
+    public void append(char src[], int off, int len) throws IOException {
         // if we have limit and we're below
         if (len <= cb.capacity() - cb.limit()) {
             transfer(src, off, len, cb);
@@ -737,7 +725,7 @@ public class OutputBuffer extends Writer {
     }
 
 
-    private void appendByteArray(byte[] src, int off, int len) throws IOException {
+    private void appendByteArray(byte src[], int off, int len) throws IOException {
         if (len == 0) {
             return;
         }
@@ -852,19 +840,4 @@ public class OutputBuffer extends Writer {
     private void toWriteMode(Buffer buffer) {
         buffer.mark().position(buffer.limit()).limit(buffer.capacity());
     }
-
-
-    private static class PrivilegedCreateConverter implements PrivilegedExceptionAction<C2BConverter> {
-
-        private final Charset charset;
-
-        PrivilegedCreateConverter(Charset charset) {
-            this.charset = charset;
-        }
-
-        @Override
-        public C2BConverter run() throws IOException {
-            return new C2BConverter(charset);
-        }
-    }
 }
