@@ -24,10 +24,6 @@ import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
 import java.nio.charset.Charset;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
-import java.security.PrivilegedActionException;
-import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -43,19 +39,18 @@ import jakarta.servlet.ServletOutputStream;
 import jakarta.servlet.ServletResponse;
 import jakarta.servlet.SessionTrackingMode;
 import jakarta.servlet.http.Cookie;
-import jakarta.servlet.http.HttpServletRequest;
 import jakarta.servlet.http.HttpServletResponse;
 import jakarta.servlet.http.HttpServletResponseWrapper;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Session;
-import org.apache.catalina.security.SecurityUtil;
 import org.apache.catalina.util.SessionConfig;
 import org.apache.coyote.ActionCode;
 import org.apache.coyote.ContinueResponseTiming;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.buf.CharChunk;
+import org.apache.tomcat.util.buf.CharsetHolder;
 import org.apache.tomcat.util.buf.UEncoder;
 import org.apache.tomcat.util.buf.UEncoder.SafeCharsSet;
 import org.apache.tomcat.util.buf.UriUtil;
@@ -80,49 +75,10 @@ public class Response implements HttpServletResponse {
 
     protected static final int SC_EARLY_HINTS = 103;
 
-    // ----------------------------------------------------- Instance Variables
-
-    public Response() {
-        this(OutputBuffer.DEFAULT_BUFFER_SIZE);
-    }
-
-
-    public Response(int outputBufferSize) {
-        outputBuffer = new OutputBuffer(outputBufferSize);
-    }
-
-
-    // ------------------------------------------------------------- Properties
-
     /**
      * Coyote response.
      */
-    protected org.apache.coyote.Response coyoteResponse;
-
-    /**
-     * Set the Coyote response.
-     *
-     * @param coyoteResponse The Coyote response
-     */
-    public void setCoyoteResponse(org.apache.coyote.Response coyoteResponse) {
-        this.coyoteResponse = coyoteResponse;
-        outputBuffer.setResponse(coyoteResponse);
-    }
-
-    /**
-     * @return the Coyote response.
-     */
-    public org.apache.coyote.Response getCoyoteResponse() {
-        return this.coyoteResponse;
-    }
-
-
-    /**
-     * @return the Context within which this Request is being processed.
-     */
-    public Context getContext() {
-        return request.getContext();
-    }
+    protected final org.apache.coyote.Response coyoteResponse;
 
 
     /**
@@ -185,23 +141,43 @@ public class Response implements HttpServletResponse {
     protected final CharChunk redirectURLCC = new CharChunk();
 
 
-    /*
-     * Not strictly required but it makes generating HTTP/2 push requests a lot easier if these are retained until the
-     * response is recycled.
-     */
-    private final List<Cookie> cookies = new ArrayList<>();
-
     private HttpServletResponse applicationResponse = null;
 
 
+    public Response(org.apache.coyote.Response coyoteResponse) {
+        this(coyoteResponse, OutputBuffer.DEFAULT_BUFFER_SIZE);
+    }
+
+
+    public Response(org.apache.coyote.Response coyoteResponse, int outputBufferSize) {
+        this.coyoteResponse = coyoteResponse;
+        outputBuffer = new OutputBuffer(outputBufferSize, coyoteResponse);
+    }
+
+
     // --------------------------------------------------------- Public Methods
 
+    /**
+     * @return the Coyote response.
+     */
+    public org.apache.coyote.Response getCoyoteResponse() {
+        return this.coyoteResponse;
+    }
+
+
+    /**
+     * @return the Context within which this Request is being processed.
+     */
+    public Context getContext() {
+        return request.getContext();
+    }
+
+
     /**
      * Release all object references, and initialize instance variables, in preparation for reuse of this object.
      */
     public void recycle() {
 
-        cookies.clear();
         outputBuffer.recycle();
         usingOutputStream = false;
         usingWriter = false;
@@ -230,11 +206,6 @@ public class Response implements HttpServletResponse {
     }
 
 
-    public List<Cookie> getCookies() {
-        return cookies;
-    }
-
-
     // ------------------------------------------------------- Response Methods
 
     /**
@@ -376,15 +347,10 @@ public class Response implements HttpServletResponse {
 
 
     /**
-     * Set the error flag.
-     *
-     * @return <code>false</code> if the error flag was already set
-     *
-     * @deprecated This method will be changed to return void in Tomcat 11 onwards
+     * Set the error flag if not already set.
      */
-    @Deprecated
-    public boolean setError() {
-        return getCoyoteResponse().setError();
+    public void setError() {
+        getCoyoteResponse().setError();
     }
 
 
@@ -478,22 +444,19 @@ public class Response implements HttpServletResponse {
 
     @Override
     public String getCharacterEncoding() {
-        String charset = getCoyoteResponse().getCharacterEncoding();
-        if (charset != null) {
-            return charset;
+        String charset = getCoyoteResponse().getCharsetHolder().getName();
+        if (charset == null) {
+            Context context = getContext();
+            if (context != null) {
+                charset = context.getResponseCharacterEncoding();
+            }
         }
 
-        Context context = getContext();
-        String result = null;
-        if (context != null) {
-            result = context.getResponseCharacterEncoding();
+        if (charset == null) {
+            charset = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET.name();
         }
 
-        if (result == null) {
-            result = org.apache.coyote.Constants.DEFAULT_BODY_CHARSET.name();
-        }
-
-        return result;
+        return charset;
     }
 
 
@@ -648,11 +611,7 @@ public class Response implements HttpServletResponse {
 
         if (type == null) {
             getCoyoteResponse().setContentType(null);
-            try {
-                getCoyoteResponse().setCharacterEncoding(null);
-            } catch (UnsupportedEncodingException e) {
-                // Can never happen when calling with null
-            }
+            getCoyoteResponse().setCharsetHolder(CharsetHolder.EMPTY);
             isCharacterEncodingSet = false;
             return;
         }
@@ -678,8 +637,9 @@ public class Response implements HttpServletResponse {
 
             // Ignore charset if getWriter() has already been called
             if (!usingWriter) {
+                getCoyoteResponse().setCharsetHolder(CharsetHolder.getInstance(m[1]));
                 try {
-                    getCoyoteResponse().setCharacterEncoding(m[1]);
+                    getCoyoteResponse().getCharsetHolder().validate();
                 } catch (UnsupportedEncodingException e) {
                     log.warn(sm.getString("coyoteResponse.encoding.invalid", m[1]), e);
                 }
@@ -708,8 +668,9 @@ public class Response implements HttpServletResponse {
             return;
         }
 
+        getCoyoteResponse().setCharsetHolder(CharsetHolder.getInstance(encoding));
         try {
-            getCoyoteResponse().setCharacterEncoding(encoding);
+            getCoyoteResponse().getCharsetHolder().validate();
         } catch (UnsupportedEncodingException e) {
             log.warn(sm.getString("coyoteResponse.encoding.invalid", encoding), e);
             return;
@@ -722,6 +683,33 @@ public class Response implements HttpServletResponse {
     }
 
 
+    @Override
+    public void setCharacterEncoding(Charset charset) {
+
+        if (isCommitted()) {
+            return;
+        }
+
+        // Ignore any call from an included servlet
+        if (included) {
+            return;
+        }
+
+        // Ignore any call made after the getWriter has been invoked
+        // The default should be used
+        if (usingWriter) {
+            return;
+        }
+
+        getCoyoteResponse().setCharsetHolder(CharsetHolder.getInstance(charset));
+        if (charset == null) {
+            isCharacterEncodingSet = false;
+        } else {
+            isCharacterEncodingSet = true;
+        }
+    }
+
+
     @Override
     public void setLocale(Locale locale) {
 
@@ -747,11 +735,7 @@ public class Response implements HttpServletResponse {
         }
 
         if (locale == null) {
-            try {
-                getCoyoteResponse().setCharacterEncoding(null);
-            } catch (UnsupportedEncodingException e) {
-                // Impossible when calling with null
-            }
+            getCoyoteResponse().setCharsetHolder(CharsetHolder.EMPTY);
         } else {
             // In some error handling scenarios, the context is unknown
             // (e.g. a 404 when a ROOT context is not present)
@@ -759,8 +743,9 @@ public class Response implements HttpServletResponse {
             if (context != null) {
                 String charset = context.getCharset(locale);
                 if (charset != null) {
+                    getCoyoteResponse().setCharsetHolder(CharsetHolder.getInstance(charset));
                     try {
-                        getCoyoteResponse().setCharacterEncoding(charset);
+                        getCoyoteResponse().getCharsetHolder().validate();
                     } catch (UnsupportedEncodingException e) {
                         log.warn(sm.getString("coyoteResponse.encoding.invalid", charset), e);
                     }
@@ -832,8 +817,6 @@ public class Response implements HttpServletResponse {
             return;
         }
 
-        cookies.add(cookie);
-
         String header = generateCookieString(cookie);
         // if we reached here, no exception, cookie is valid
         addHeader("Set-Cookie", header, getContext().getCookieProcessor().getCharset());
@@ -874,12 +857,7 @@ public class Response implements HttpServletResponse {
     public String generateCookieString(final Cookie cookie) {
         // Web application code can receive a IllegalArgumentException
         // from the generateHeader() invocation
-        if (SecurityUtil.isPackageProtectionEnabled()) {
-            return AccessController
-                    .doPrivileged(new PrivilegedGenerateCookieString(getContext(), cookie, request.getRequest()));
-        } else {
-            return getContext().getCookieProcessor().generateHeader(cookie, request.getRequest());
-        }
+        return getContext().getCookieProcessor().generateHeader(cookie, request.getRequest());
     }
 
 
@@ -1045,10 +1023,8 @@ public class Response implements HttpServletResponse {
      *
      * @param continueResponseTiming Indicates when the request for the ACK originated so it can be compared with the
      *                                   configured timing for ACK responses.
-     *
-     * @exception IOException if an input/output error occurs
      */
-    public void sendAcknowledgement(ContinueResponseTiming continueResponseTiming) throws IOException {
+    public void sendAcknowledgement(ContinueResponseTiming continueResponseTiming) {
 
         if (isCommitted()) {
             return;
@@ -1133,21 +1109,7 @@ public class Response implements HttpServletResponse {
 
 
     @Override
-    public void sendRedirect(String location) throws IOException {
-        sendRedirect(location, SC_FOUND);
-    }
-
-
-    /**
-     * Internal method that allows a redirect to be sent with a status other than {@link HttpServletResponse#SC_FOUND}
-     * (302). No attempt is made to validate the status code.
-     *
-     * @param location Location URL to redirect to
-     * @param status   HTTP status code that will be sent
-     *
-     * @throws IOException an IO exception occurred
-     */
-    public void sendRedirect(String location, int status) throws IOException {
+    public void sendRedirect(String location, int status, boolean clearBuffer) throws IOException {
         if (isCommitted()) {
             throw new IllegalStateException(sm.getString("coyoteResponse.sendRedirect.ise"));
         }
@@ -1158,7 +1120,9 @@ public class Response implements HttpServletResponse {
         }
 
         // Clear any data content that has been buffered
-        resetBuffer(true);
+        if (clearBuffer) {
+            resetBuffer(true);
+        }
 
         // Generate a temporary redirect to the specified location
         try {
@@ -1176,7 +1140,7 @@ public class Response implements HttpServletResponse {
             }
             setStatus(status);
             setHeader("Location", locationUri);
-            if (context != null && context.getSendRedirectBody()) {
+            if (clearBuffer && context != null && context.getSendRedirectBody()) {
                 PrintWriter writer = getWriter();
                 writer.print(sm.getString("coyoteResponse.sendRedirect.note", Escape.htmlElementContent(locationUri)));
                 flushBuffer();
@@ -1214,7 +1178,7 @@ public class Response implements HttpServletResponse {
     @Override
     public void setHeader(String name, String value) {
 
-        if (name == null || name.length() == 0 || value == null) {
+        if (name == null || name.length() == 0) {
             return;
         }
 
@@ -1316,13 +1280,7 @@ public class Response implements HttpServletResponse {
             return false;
         }
 
-        if (SecurityUtil.isPackageProtectionEnabled()) {
-            Boolean result =
-                    AccessController.doPrivileged(new PrivilegedDoIsEncodable(getContext(), hreq, session, location));
-            return result.booleanValue();
-        } else {
-            return doIsEncodeable(getContext(), hreq, session, location);
-        }
+        return doIsEncodeable(getContext(), hreq, session, location);
     }
 
 
@@ -1434,17 +1392,7 @@ public class Response implements HttpServletResponse {
                 if (!leadingSlash) {
                     String relativePath = request.getDecodedRequestURI();
                     int pos = relativePath.lastIndexOf('/');
-                    CharChunk encodedURI = null;
-                    if (SecurityUtil.isPackageProtectionEnabled()) {
-                        try {
-                            encodedURI = AccessController
-                                    .doPrivileged(new PrivilegedEncodeUrl(urlEncoder, relativePath, pos));
-                        } catch (PrivilegedActionException pae) {
-                            throw new IllegalArgumentException(location, pae.getException());
-                        }
-                    } else {
-                        encodedURI = urlEncoder.encodeURL(relativePath, 0, pos);
-                    }
+                    CharChunk encodedURI = urlEncoder.encodeURL(relativePath, 0, pos);
                     redirectURLCC.append(encodedURI);
                     encodedURI.recycle();
                     redirectURLCC.append('/');
@@ -1612,63 +1560,4 @@ public class Response implements HttpServletResponse {
         sb.append(query);
         return sb.toString();
     }
-
-
-    private static class PrivilegedGenerateCookieString implements PrivilegedAction<String> {
-
-        private final Context context;
-        private final Cookie cookie;
-        private final HttpServletRequest request;
-
-        PrivilegedGenerateCookieString(Context context, Cookie cookie, HttpServletRequest request) {
-            this.context = context;
-            this.cookie = cookie;
-            this.request = request;
-        }
-
-        @Override
-        public String run() {
-            return context.getCookieProcessor().generateHeader(cookie, request);
-        }
-    }
-
-
-    private static class PrivilegedDoIsEncodable implements PrivilegedAction<Boolean> {
-
-        private final Context context;
-        private final Request hreq;
-        private final Session session;
-        private final String location;
-
-        PrivilegedDoIsEncodable(Context context, Request hreq, Session session, String location) {
-            this.context = context;
-            this.hreq = hreq;
-            this.session = session;
-            this.location = location;
-        }
-
-        @Override
-        public Boolean run() {
-            return Boolean.valueOf(doIsEncodeable(context, hreq, session, location));
-        }
-    }
-
-
-    private static class PrivilegedEncodeUrl implements PrivilegedExceptionAction<CharChunk> {
-
-        private final UEncoder urlEncoder;
-        private final String relativePath;
-        private final int end;
-
-        PrivilegedEncodeUrl(UEncoder urlEncoder, String relativePath, int end) {
-            this.urlEncoder = urlEncoder;
-            this.relativePath = relativePath;
-            this.end = end;
-        }
-
-        @Override
-        public CharChunk run() throws IOException {
-            return urlEncoder.encodeURL(relativePath, 0, end);
-        }
-    }
 }
