@@ -161,7 +161,7 @@ public class ApplicationContext implements ServletContext {
     /**
      * Session Cookie config
      */
-    private final SessionCookieConfig sessionCookieConfig;
+    private SessionCookieConfig sessionCookieConfig;
 
     /**
      * Session tracking modes
@@ -200,7 +200,7 @@ public class ApplicationContext implements ServletContext {
             return null;
         }
 
-        Context child;
+        Context child = null;
         try {
             // Look for an exact match
             Container host = context.getParent();
@@ -314,7 +314,7 @@ public class ApplicationContext implements ServletContext {
             return null;
         }
         String extension = file.substring(period + 1);
-        if (extension.isEmpty()) {
+        if (extension.length() < 1) {
             return null;
         }
         return context.findMimeMapping(extension);
@@ -372,32 +372,36 @@ public class ApplicationContext implements ServletContext {
             queryString = null;
         }
 
-        // From this point, the removal of path parameters, decoding and normalization is only for mapping purposes.
         // Remove path parameters
-        String uriToMap = org.apache.catalina.util.RequestUtil.stripPathParams(uri, null);
-
-        // Decode only if the uri derived from the provided path is expected to be encoded
-        if (getContext().getDispatchersUseEncodedPaths()) {
-            uriToMap = UDecoder.URLDecode(uriToMap, StandardCharsets.UTF_8, context.getEncodedSolidusHandlingEnum(),
-                    context.getEncodedReverseSolidusHandlingEnum());
-        }
+        String uriNoParams = stripPathParams(uri);
 
         // Then normalize
-        uriToMap = RequestUtil.normalize(uriToMap);
-        if (uriToMap == null) {
-            getContext().getLogger().warn(sm.getString("applicationContext.illegalDispatchPath", path),
-                    new IllegalArgumentException());
+        String normalizedUri = RequestUtil.normalize(uriNoParams);
+        if (normalizedUri == null) {
             return null;
         }
 
-        /*
-         * uri is passed to the constructor for ApplicationDispatcher and is ultimately used as the value for
-         * getRequestURI() which returns encoded values. getContextPath() returns a decoded value. uri may be encoded or
-         * not. Need to prepend the context path to uri and ensure the result is correctly encoded.
-         */
+        // Mapping is against the normalized uri
+
         if (getContext().getDispatchersUseEncodedPaths()) {
+            // Decode
+            String decodedUri = UDecoder.URLDecode(normalizedUri, StandardCharsets.UTF_8);
+
+            // Security check to catch attempts to encode /../ sequences
+            normalizedUri = RequestUtil.normalize(decodedUri);
+            if (!decodedUri.equals(normalizedUri)) {
+                getContext().getLogger().warn(sm.getString("applicationContext.illegalDispatchPath", path),
+                        new IllegalArgumentException());
+                return null;
+            }
+
+            // URI needs to include the context path
             uri = URLEncoder.DEFAULT.encode(getContextPath(), StandardCharsets.UTF_8) + uri;
         } else {
+            // uri is passed to the constructor for ApplicationDispatcher and is
+            // ultimately used as the value for getRequestURI() which returns
+            // encoded values. Therefore, since the value passed in for path
+            // was decoded, encode uri here.
             uri = URLEncoder.DEFAULT.encode(getContextPath() + uri, StandardCharsets.UTF_8);
         }
 
@@ -418,7 +422,7 @@ public class ApplicationContext implements ServletContext {
             CharChunk uriCC = uriMB.getCharChunk();
             try {
                 uriCC.append(context.getPath());
-                uriCC.append(uriToMap);
+                uriCC.append(normalizedUri);
                 service.getMapper().map(context, uriMB, mappingData);
                 if (mappingData.wrapper == null) {
                     return null;
@@ -449,6 +453,34 @@ public class ApplicationContext implements ServletContext {
     }
 
 
+    // Package private to facilitate testing
+    static String stripPathParams(String input) {
+        // Shortcut
+        if (input.indexOf(';') < 0) {
+            return input;
+        }
+
+        StringBuilder sb = new StringBuilder(input.length());
+        int pos = 0;
+        int limit = input.length();
+        while (pos < limit) {
+            int nextSemiColon = input.indexOf(';', pos);
+            if (nextSemiColon < 0) {
+                nextSemiColon = limit;
+            }
+            sb.append(input.substring(pos, nextSemiColon));
+            int followingSlash = input.indexOf('/', nextSemiColon);
+            if (followingSlash < 0) {
+                pos = limit;
+            } else {
+                pos = followingSlash;
+            }
+        }
+
+        return sb.toString();
+    }
+
+
     @Override
     public URL getResource(String path) throws MalformedURLException {
 
@@ -552,18 +584,20 @@ public class ApplicationContext implements ServletContext {
     @Override
     public void removeAttribute(String name) {
 
+        Object value = null;
+
         // Remove the specified attribute
         // Check for read only attribute
         if (readOnlyAttributes.containsKey(name)) {
             return;
         }
-        Object value = attributes.remove(name);
+        value = attributes.remove(name);
         if (value == null) {
             return;
         }
 
         // Notify interested application event listeners
-        Object[] listeners = context.getApplicationEventListeners();
+        Object listeners[] = context.getApplicationEventListeners();
         if (listeners == null || listeners.length == 0) {
             return;
         }
@@ -609,11 +643,11 @@ public class ApplicationContext implements ServletContext {
         boolean replaced = oldValue != null;
 
         // Notify interested application event listeners
-        Object[] listeners = context.getApplicationEventListeners();
+        Object listeners[] = context.getApplicationEventListeners();
         if (listeners == null || listeners.length == 0) {
             return;
         }
-        ServletContextAttributeEvent event;
+        ServletContextAttributeEvent event = null;
         if (replaced) {
             event = new ServletContextAttributeEvent(context.getServletContext(), name, oldValue);
         } else {
@@ -669,7 +703,7 @@ public class ApplicationContext implements ServletContext {
     private FilterRegistration.Dynamic addFilter(String filterName, String filterClass, Filter filter)
             throws IllegalStateException {
 
-        if (filterName == null || filterName.isEmpty()) {
+        if (filterName == null || filterName.equals("")) {
             throw new IllegalArgumentException(sm.getString("applicationContext.invalidFilterName", filterName));
         }
 
@@ -752,11 +786,11 @@ public class ApplicationContext implements ServletContext {
             throw new IllegalArgumentException(sm.getString("applicationContext.addJspFile.iae", jspFile));
         }
 
+        String jspServletClassName = null;
         Map<String,String> jspFileInitParams = new HashMap<>();
 
         Wrapper jspServlet = (Wrapper) context.findChild("jsp");
 
-        String jspServletClassName;
         if (jspServlet == null) {
             // No JSP servlet currently defined.
             // Use default JSP Servlet class name
@@ -782,7 +816,7 @@ public class ApplicationContext implements ServletContext {
     private ServletRegistration.Dynamic addServlet(String servletName, String servletClass, Servlet servlet,
             Map<String,String> initParams) throws IllegalStateException {
 
-        if (servletName == null || servletName.isEmpty()) {
+        if (servletName == null || servletName.equals("")) {
             throw new IllegalArgumentException(sm.getString("applicationContext.invalidServletName", servletName));
         }
 
@@ -879,7 +913,7 @@ public class ApplicationContext implements ServletContext {
             supportedSessionTrackingModes.add(SessionTrackingMode.COOKIE);
         }
 
-        // SSL not enabled by default as it can only be used on its own
+        // SSL not enabled by default as it can only used on its own
         // Context > Host > Engine > Service
         Connector[] connectors = service.findConnectors();
         // Need at least one SSL enabled connector to use the SSL session ID.
@@ -920,7 +954,7 @@ public class ApplicationContext implements ServletContext {
             }
         }
 
-        // Check SSL has not been configured with anything else
+        // Check SSL has not be configured with anything else
         if (sessionTrackingModes.contains(SessionTrackingMode.SSL)) {
             if (sessionTrackingModes.size() > 1) {
                 throw new IllegalArgumentException(
@@ -1058,22 +1092,7 @@ public class ApplicationContext implements ServletContext {
 
     @Override
     public ClassLoader getClassLoader() {
-        ClassLoader result = context.getLoader().getClassLoader();
-        if (Globals.IS_SECURITY_ENABLED) {
-            ClassLoader tccl = Thread.currentThread().getContextClassLoader();
-            ClassLoader parent = result;
-            while (parent != null) {
-                if (parent == tccl) {
-                    break;
-                }
-                parent = parent.getParent();
-            }
-            if (parent == null) {
-                System.getSecurityManager().checkPermission(new RuntimePermission("getClassLoader"));
-            }
-        }
-
-        return result;
+        return context.getLoader().getClassLoader();
     }
 
 
