@@ -25,7 +25,6 @@ import java.net.InetAddress;
 import java.net.ServerSocket;
 import java.net.Socket;
 import java.net.SocketTimeoutException;
-import java.security.AccessControlException;
 import java.util.Random;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ScheduledExecutorService;
@@ -62,6 +61,8 @@ import org.apache.tomcat.util.threads.TaskThreadFactory;
 /**
  * Standard implementation of the <b>Server</b> interface, available for use (but not required) when deploying and
  * starting Catalina.
+ *
+ * @author Craig R. McClanahan
  */
 public final class StandardServer extends LifecycleMBeanBase implements Server {
 
@@ -103,7 +104,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
     /**
      * Global naming resources.
      */
-    private NamingResourcesImpl globalNamingResources;
+    private NamingResourcesImpl globalNamingResources = null;
 
 
     /**
@@ -460,7 +461,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
 
         servicesWriteLock.lock();
         try {
-            Service[] results = new Service[services.length + 1];
+            Service results[] = new Service[services.length + 1];
             System.arraycopy(services, 0, results, 0, services.length);
             results[services.length] = service;
             services = results;
@@ -489,14 +490,14 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
                 awaitSocket = null;
                 try {
                     s.close();
-                } catch (IOException ignore) {
+                } catch (IOException e) {
                     // Ignored
                 }
             }
             t.interrupt();
             try {
                 t.join(1000);
-            } catch (InterruptedException ignore) {
+            } catch (InterruptedException e) {
                 // Ignored
             }
         }
@@ -529,9 +530,9 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         // Set up a server socket to wait on
         try {
             awaitSocket = new ServerSocket(getPortWithOffset(), 1, InetAddress.getByName(address));
-        } catch (IOException ioe) {
+        } catch (IOException e) {
             log.error(sm.getString("standardServer.awaitSocket.fail", address, String.valueOf(getPortWithOffset()),
-                    String.valueOf(getPort()), String.valueOf(getPortOffset())), ioe);
+                    String.valueOf(getPort()), String.valueOf(getPortOffset())), e);
             return;
         }
 
@@ -561,15 +562,12 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
                         log.warn(sm.getString("standardServer.accept.timeout",
                                 Long.valueOf(System.currentTimeMillis() - acceptStartTime)), ste);
                         continue;
-                    } catch (AccessControlException ace) {
-                        log.warn(sm.getString("standardServer.accept.security"), ace);
-                        continue;
-                    } catch (IOException ioe) {
+                    } catch (IOException e) {
                         if (stopAwait) {
                             // Wait was aborted with socket.close()
                             break;
                         }
-                        log.error(sm.getString("standardServer.accept.error"), ioe);
+                        log.error(sm.getString("standardServer.accept.error"), e);
                         break;
                     }
 
@@ -582,11 +580,11 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
                         expected += random.nextInt() % 1024;
                     }
                     while (expected > 0) {
-                        int ch;
+                        int ch = -1;
                         try {
                             ch = stream.read();
-                        } catch (IOException ioe) {
-                            log.warn(sm.getString("standardServer.accept.readError"), ioe);
+                        } catch (IOException e) {
+                            log.warn(sm.getString("standardServer.accept.readError"), e);
                             ch = -1;
                         }
                         // Control character or EOF (-1) terminates loop
@@ -602,7 +600,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
                         if (socket != null) {
                             socket.close();
                         }
-                    } catch (IOException ignore) {
+                    } catch (IOException e) {
                         // Ignore
                     }
                 }
@@ -625,7 +623,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
             if (serverSocket != null) {
                 try {
                     serverSocket.close();
-                } catch (IOException ignore) {
+                } catch (IOException e) {
                     // Ignore
                 }
             }
@@ -795,7 +793,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         try {
             // Note: Hard-coded domain used since this object is per Server/JVM
             ObjectName sname = new ObjectName("Catalina:type=StoreConfig");
-            MBeanServer server = Registry.getRegistry(null).getMBeanServer();
+            MBeanServer server = Registry.getRegistry(null, null).getMBeanServer();
             if (server.isRegistered(sname)) {
                 server.invoke(sname, "storeConfig", null, null);
             } else {
@@ -822,7 +820,7 @@ public final class StandardServer extends LifecycleMBeanBase implements Server {
         try {
             // Note: Hard-coded domain used since this object is per Server/JVM
             ObjectName sname = new ObjectName("Catalina:type=StoreConfig");
-            MBeanServer server = Registry.getRegistry(null).getMBeanServer();
+            MBeanServer server = Registry.getRegistry(null, null).getMBeanServer();
             if (server.isRegistered(sname)) {
                 server.invoke(sname, "store", new Object[] { context }, new String[] { "java.lang.String" });
             } else {
