@@ -208,7 +208,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
         String ejbLink = ejb.getLink();
         String lookupName = ejb.getLookupName();
 
-        if (ejbLink != null && !ejbLink.isEmpty() && lookupName != null && !lookupName.isEmpty()) {
+        if (ejbLink != null && ejbLink.length() > 0 && lookupName != null && lookupName.length() > 0) {
             throw new IllegalArgumentException(sm.getString("namingResources.ejbLookupLink", ejb.getName()));
         }
 
@@ -260,12 +260,12 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
         String lookupName = environment.getLookupName();
 
         // Entries with injection targets but no value are effectively ignored
-        if (injectionTargets != null && !injectionTargets.isEmpty() && (value == null || value.isEmpty())) {
+        if (injectionTargets != null && injectionTargets.size() > 0 && (value == null || value.length() == 0)) {
             return;
         }
 
         // Entries with lookup-name and value are an error (EE.5.4.1.3)
-        if (value != null && !value.isEmpty() && lookupName != null && !lookupName.isEmpty()) {
+        if (value != null && value.length() > 0 && lookupName != null && lookupName.length() > 0) {
             throw new IllegalArgumentException(
                     sm.getString("namingResources.envEntryLookupValue", environment.getName()));
         }
@@ -513,7 +513,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
 
     /**
-     * @return the array of defined environment entries for this web application. If none have been defined, a zero-length
+     * @return the set of defined environment entries for this web application. If none have been defined, a zero-length
      *             array is returned.
      */
     public ContextEnvironment[] findEnvironments() {
@@ -649,7 +649,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
 
     /**
-     * @return the array of resource environment reference names for this web application. If none have been specified, a
+     * @return the set of resource environment reference names for this web application. If none have been specified, a
      *             zero-length array is returned.
      */
     public ContextResourceEnvRef[] findResourceEnvRefs() {
@@ -697,7 +697,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextEjb ejb;
+        ContextEjb ejb = null;
         synchronized (ejbs) {
             ejb = ejbs.remove(name);
         }
@@ -714,7 +714,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextEnvironment environment;
+        ContextEnvironment environment = null;
         synchronized (envs) {
             environment = envs.remove(name);
         }
@@ -742,7 +742,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextLocalEjb localEjb;
+        ContextLocalEjb localEjb = null;
         synchronized (localEjbs) {
             localEjb = localEjbs.remove(name);
         }
@@ -763,7 +763,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        MessageDestinationRef mdr;
+        MessageDestinationRef mdr = null;
         synchronized (mdrs) {
             mdr = mdrs.remove(name);
         }
@@ -792,7 +792,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextResource resource;
+        ContextResource resource = null;
         synchronized (resources) {
             resource = resources.remove(name);
         }
@@ -820,7 +820,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextResourceEnvRef resourceEnvRef;
+        ContextResourceEnvRef resourceEnvRef = null;
         synchronized (resourceEnvRefs) {
             resourceEnvRef = resourceEnvRefs.remove(name);
         }
@@ -837,7 +837,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextResourceLink resourceLink;
+        ContextResourceLink resourceLink = null;
         synchronized (resourceLinks) {
             resourceLink = resourceLinks.remove(name);
         }
@@ -865,7 +865,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
 
         entries.remove(name);
 
-        ContextService service;
+        ContextService service = null;
         synchronized (services) {
             service = services.remove(name);
         }
@@ -931,7 +931,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
      * Close those resources that an explicit close may help clean-up faster.
      */
     private void cleanUp() {
-        if (resources.isEmpty()) {
+        if (resources.size() == 0) {
             return;
         }
         javax.naming.Context ctxt;
@@ -949,7 +949,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
         for (ContextResource cr : resources.values()) {
             if (cr.getSingleton()) {
                 String closeMethod = cr.getCloseMethod();
-                if (closeMethod != null && !closeMethod.isEmpty()) {
+                if (closeMethod != null && closeMethod.length() > 0) {
                     String name = cr.getName();
                     Object resource;
                     try {
@@ -973,7 +973,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
      */
     private void cleanUp(Object resource, String name, String closeMethod) {
         // Look for a zero-arg close() method
-        Method m;
+        Method m = null;
         try {
             m = resource.getClass().getMethod(closeMethod, (Class<?>[]) null);
         } catch (SecurityException e) {
@@ -1069,7 +1069,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
             return true;
         }
 
-        if (resource.getInjectionTargets() == null || resource.getInjectionTargets().isEmpty()) {
+        if (resource.getInjectionTargets() == null || resource.getInjectionTargets().size() == 0) {
             // No injection targets so use the defined type for the resource
             return true;
         }
@@ -1151,7 +1151,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
     }
 
     private Class<?> getSetterType(Class<?> clazz, String name) {
-        Method[] methods = Introspection.getDeclaredMethods(clazz);
+        Method[] methods = clazz.getDeclaredMethods();
         if (methods != null && methods.length > 0) {
             for (Method method : methods) {
                 if (Introspection.isValidSetter(method) && Introspection.getPropertyName(method).equals(name)) {
@@ -1163,7 +1163,7 @@ public class NamingResourcesImpl extends LifecycleMBeanBase implements Serializa
     }
 
     private Class<?> getFieldType(Class<?> clazz, String name) {
-        Field[] fields = Introspection.getDeclaredFields(clazz);
+        Field[] fields = clazz.getDeclaredFields();
         if (fields != null && fields.length > 0) {
             for (Field field : fields) {
                 if (field.getName().equals(name)) {
