@@ -40,6 +40,8 @@ import org.apache.tomcat.util.security.Escape;
 /**
  * This is a refactoring of the servlet to externalize the output into a simple class. Although we could use XSLT, that
  * is unnecessarily complex.
+ *
+ * @author Peter Lin
  */
 public class StatusTransformer {
 
@@ -212,9 +214,9 @@ public class StatusTransformer {
             indent(writer, 1).append('"').append("jvm").append('"').append(':').append('{').println();
 
             indent(writer, 2).append('"').append("memory").append('"').append(':').append('{');
-            appendJSonValue(writer, "free", Long.valueOf(Runtime.getRuntime().freeMemory())).append(',');
-            appendJSonValue(writer, "total", Long.valueOf(Runtime.getRuntime().totalMemory())).append(',');
-            appendJSonValue(writer, "max", Long.valueOf(Runtime.getRuntime().maxMemory()));
+            appendJSonValue(writer, "free", Long.toString(Runtime.getRuntime().freeMemory())).append(',');
+            appendJSonValue(writer, "total", Long.toString(Runtime.getRuntime().totalMemory())).append(',');
+            appendJSonValue(writer, "max", Long.toString(Runtime.getRuntime().maxMemory()));
             writer.append('}').append(',').println();
 
             indent(writer, 2).append('"').append("memorypool").append('"').append(':').append('[');
@@ -230,10 +232,10 @@ public class StatusTransformer {
                 indent(writer, 3).append('{');
                 appendJSonValue(writer, "name", JSONFilter.escape(memoryPoolMBean.getName())).append(',');
                 appendJSonValue(writer, "type", memoryPoolMBean.getType().toString()).append(',');
-                appendJSonValue(writer, "usageInit", Long.valueOf(usage.getInit())).append(',');
-                appendJSonValue(writer, "usageCommitted", Long.valueOf(usage.getCommitted())).append(',');
-                appendJSonValue(writer, "usageMax", Long.valueOf(usage.getMax())).append(',');
-                appendJSonValue(writer, "usageUsed", Long.valueOf(usage.getUsed()));
+                appendJSonValue(writer, "usageInit", Long.toString(usage.getInit())).append(',');
+                appendJSonValue(writer, "usageCommitted", Long.toString(usage.getCommitted())).append(',');
+                appendJSonValue(writer, "usageMax", Long.toString(usage.getMax())).append(',');
+                appendJSonValue(writer, "usageUsed", Long.toString(usage.getUsed()));
                 writer.append('}');
             }
             writer.println();
@@ -245,13 +247,8 @@ public class StatusTransformer {
     }
 
 
-    private static PrintWriter appendJSonValue(PrintWriter writer, String name, Object value) {
-        writer.append('"').append(name).append('"').append(':');
-        if (value instanceof Number) {
-            return writer.append(value.toString());
-        } else {
-            return writer.append('"').append(value.toString()).append('"');
-        }
+    private static PrintWriter appendJSonValue(PrintWriter writer, String name, String value) {
+        return writer.append('"').append(name).append('"').append(':').append('"').append(value).append('"');
     }
 
 
@@ -449,10 +446,12 @@ public class StatusTransformer {
             }
             appendJSonValue(writer, "name", jsonName).append(',').println();
             indent(writer, 3).append('"').append("threadInfo").append('"').append(':').append('{');
-            appendJSonValue(writer, "maxThreads", mBeanServer.getAttribute(tpName, "maxThreads")).append(',');
-            appendJSonValue(writer, "currentThreadCount", mBeanServer.getAttribute(tpName, "currentThreadCount"))
+            appendJSonValue(writer, "maxThreads", mBeanServer.getAttribute(tpName, "maxThreads").toString())
                     .append(',');
-            appendJSonValue(writer, "currentThreadsBusy", mBeanServer.getAttribute(tpName, "currentThreadsBusy"));
+            appendJSonValue(writer, "currentThreadCount",
+                    mBeanServer.getAttribute(tpName, "currentThreadCount").toString()).append(',');
+            appendJSonValue(writer, "currentThreadsBusy",
+                    mBeanServer.getAttribute(tpName, "currentThreadsBusy").toString());
             writer.append('}');
 
             ObjectName grpName = null;
@@ -466,14 +465,16 @@ public class StatusTransformer {
             if (grpName != null) {
                 writer.append(',').println();
                 indent(writer, 3).append('"').append("requestInfo").append('"').append(':').append('{');
-                appendJSonValue(writer, "maxTime", mBeanServer.getAttribute(grpName, "maxTime")).append(',');
-                appendJSonValue(writer, "processingTime", mBeanServer.getAttribute(grpName, "processingTime"))
+                appendJSonValue(writer, "maxTime", mBeanServer.getAttribute(grpName, "maxTime").toString()).append(',');
+                appendJSonValue(writer, "processingTime",
+                        mBeanServer.getAttribute(grpName, "processingTime").toString()).append(',');
+                appendJSonValue(writer, "requestCount", mBeanServer.getAttribute(grpName, "requestCount").toString())
                         .append(',');
-                appendJSonValue(writer, "requestCount", mBeanServer.getAttribute(grpName, "requestCount")).append(',');
-                appendJSonValue(writer, "errorCount", mBeanServer.getAttribute(grpName, "errorCount")).append(',');
-                appendJSonValue(writer, "bytesReceived", mBeanServer.getAttribute(grpName, "bytesReceived"))
+                appendJSonValue(writer, "errorCount", mBeanServer.getAttribute(grpName, "errorCount").toString())
                         .append(',');
-                appendJSonValue(writer, "bytesSent", mBeanServer.getAttribute(grpName, "bytesSent"));
+                appendJSonValue(writer, "bytesReceived", mBeanServer.getAttribute(grpName, "bytesReceived").toString())
+                        .append(',');
+                appendJSonValue(writer, "bytesSent", mBeanServer.getAttribute(grpName, "bytesSent").toString());
                 writer.append('}').println();
                 // Note: No detailed per processor info
             }
@@ -515,7 +516,7 @@ public class StatusTransformer {
         int stage = stageValue.intValue();
         boolean fullStatus = true;
         boolean showRequest = true;
-        String stageStr;
+        String stageStr = null;
 
         switch (stage) {
 
@@ -593,7 +594,7 @@ public class StatusTransformer {
                     writer.write(' ');
                     writer.write(Escape.htmlElementContent(mBeanServer.getAttribute(pName, "currentUri")));
                     String queryString = (String) mBeanServer.getAttribute(pName, "currentQueryString");
-                    if (queryString != null && !queryString.isEmpty()) {
+                    if (queryString != null && !queryString.equals("")) {
                         writer.write("?");
                         writer.print(Escape.htmlElementContent(queryString));
                     }
@@ -639,7 +640,7 @@ public class StatusTransformer {
                             Escape.htmlElementContent(mBeanServer.getAttribute(pName, "currentUri")) + "\"");
 
                     String queryString = (String) mBeanServer.getAttribute(pName, "currentQueryString");
-                    if (queryString != null && !queryString.isEmpty()) {
+                    if (queryString != null && !queryString.equals("")) {
                         writer.write(" currentQueryString=\"" + Escape.htmlElementContent(queryString) + "\"");
                     } else {
                         writer.write(" currentQueryString=\"&#63;\"");
@@ -765,8 +766,8 @@ public class StatusTransformer {
             return;
         }
 
-        String hostName;
-        String contextName;
+        String hostName = null;
+        String contextName = null;
         if (name.startsWith("//")) {
             name = name.substring(2);
         }
@@ -803,8 +804,6 @@ public class StatusTransformer {
             writer.print("</a>");
 
             writer.print("<p>");
-            writer.print("State: ");
-            writer.print(mBeanServer.getAttribute(objectName, "stateName"));
             Object startTime = mBeanServer.getAttribute(objectName, "startTime");
             writer.print(" Start time: " + new Date(((Long) startTime).longValue()));
             writer.print(" Startup time: ");
@@ -831,13 +830,12 @@ public class StatusTransformer {
         } else if (mode == 2) {
             indent(writer, 2).append('{').println();
             appendJSonValue(indent(writer, 3), "name", JSONFilter.escape(JSONFilter.escape(name))).append(',');
-            appendJSonValue(writer, "state", mBeanServer.getAttribute(objectName, "stateName"));
-            writer.append(',');
             appendJSonValue(writer, "startTime",
                     new Date(((Long) mBeanServer.getAttribute(objectName, "startTime")).longValue()).toString())
                     .append(',');
-            appendJSonValue(writer, "startupTime", mBeanServer.getAttribute(objectName, "startupTime")).append(',');
-            appendJSonValue(writer, "tldScanTime", mBeanServer.getAttribute(objectName, "tldScanTime"));
+            appendJSonValue(writer, "startupTime", mBeanServer.getAttribute(objectName, "startupTime").toString())
+                    .append(',');
+            appendJSonValue(writer, "tldScanTime", mBeanServer.getAttribute(objectName, "tldScanTime").toString());
             if (managerON != null) {
                 writeManager(writer, managerON, mBeanServer, mode);
             }
@@ -902,20 +900,22 @@ public class StatusTransformer {
         } else if (mode == 2) {
             writer.append(',').println();
             indent(writer, 3).append('"').append("manager").append('"').append(':').append('{');
-            appendJSonValue(writer, "activeSessions", mBeanServer.getAttribute(objectName, "activeSessions"))
+            appendJSonValue(writer, "activeSessions", mBeanServer.getAttribute(objectName, "activeSessions").toString())
                     .append(',');
-            appendJSonValue(writer, "sessionCounter", mBeanServer.getAttribute(objectName, "sessionCounter"))
+            appendJSonValue(writer, "sessionCounter", mBeanServer.getAttribute(objectName, "sessionCounter").toString())
                     .append(',');
-            appendJSonValue(writer, "maxActive", mBeanServer.getAttribute(objectName, "maxActive")).append(',');
-            appendJSonValue(writer, "rejectedSessions", mBeanServer.getAttribute(objectName, "rejectedSessions"))
-                    .append(',');
-            appendJSonValue(writer, "expiredSessions", mBeanServer.getAttribute(objectName, "expiredSessions"))
-                    .append(',');
-            appendJSonValue(writer, "sessionMaxAliveTime", mBeanServer.getAttribute(objectName, "sessionMaxAliveTime"))
+            appendJSonValue(writer, "maxActive", mBeanServer.getAttribute(objectName, "maxActive").toString())
                     .append(',');
+            appendJSonValue(writer, "rejectedSessions",
+                    mBeanServer.getAttribute(objectName, "rejectedSessions").toString()).append(',');
+            appendJSonValue(writer, "expiredSessions",
+                    mBeanServer.getAttribute(objectName, "expiredSessions").toString()).append(',');
+            appendJSonValue(writer, "sessionMaxAliveTime",
+                    mBeanServer.getAttribute(objectName, "sessionMaxAliveTime").toString()).append(',');
             appendJSonValue(writer, "sessionAverageAliveTime",
-                    mBeanServer.getAttribute(objectName, "sessionAverageAliveTime")).append(',');
-            appendJSonValue(writer, "processingTime", mBeanServer.getAttribute(objectName, "processingTime"));
+                    mBeanServer.getAttribute(objectName, "sessionAverageAliveTime").toString()).append(',');
+            appendJSonValue(writer, "processingTime",
+                    mBeanServer.getAttribute(objectName, "processingTime").toString());
             writer.append('}');
         }
 
@@ -956,8 +956,8 @@ public class StatusTransformer {
         } else if (mode == 2) {
             writer.append(',').println();
             indent(writer, 3).append('"').append("jsp").append('"').append(':').append('{');
-            appendJSonValue(writer, "jspCount", Integer.valueOf(jspCount)).append(',');
-            appendJSonValue(writer, "jspReloadCount", Integer.valueOf(jspReloadCount));
+            appendJSonValue(writer, "jspCount", Integer.toString(jspCount)).append(',');
+            appendJSonValue(writer, "jspReloadCount", Integer.toString(jspReloadCount));
             writer.append('}');
         }
     }
@@ -1014,13 +1014,16 @@ public class StatusTransformer {
         } else if (mode == 2) {
             indent(writer, 4).append('{');
             appendJSonValue(writer, "servletName", JSONFilter.escape(servletName)).append(',');
-            appendJSonValue(writer, "processingTime", mBeanServer.getAttribute(objectName, "processingTime"))
+            appendJSonValue(writer, "processingTime", mBeanServer.getAttribute(objectName, "processingTime").toString())
                     .append(',');
-            appendJSonValue(writer, "maxTime", mBeanServer.getAttribute(objectName, "maxTime")).append(',');
-            appendJSonValue(writer, "requestCount", mBeanServer.getAttribute(objectName, "requestCount")).append(',');
-            appendJSonValue(writer, "errorCount", mBeanServer.getAttribute(objectName, "errorCount")).append(',');
-            appendJSonValue(writer, "loadTime", mBeanServer.getAttribute(objectName, "loadTime")).append(',');
-            appendJSonValue(writer, "classLoadTime", mBeanServer.getAttribute(objectName, "classLoadTime"));
+            appendJSonValue(writer, "maxTime", mBeanServer.getAttribute(objectName, "maxTime").toString()).append(',');
+            appendJSonValue(writer, "requestCount", mBeanServer.getAttribute(objectName, "requestCount").toString())
+                    .append(',');
+            appendJSonValue(writer, "errorCount", mBeanServer.getAttribute(objectName, "errorCount").toString())
+                    .append(',');
+            appendJSonValue(writer, "loadTime", mBeanServer.getAttribute(objectName, "loadTime").toString())
+                    .append(',');
+            appendJSonValue(writer, "classLoadTime", mBeanServer.getAttribute(objectName, "classLoadTime").toString());
             writer.append('}');
         }
 
