@@ -37,15 +37,14 @@ import javax.naming.NamingException;
 import javax.sql.DataSource;
 
 import org.apache.catalina.Container;
-import org.apache.catalina.Globals;
-import org.apache.catalina.Server;
-import org.apache.catalina.Service;
 import org.apache.catalina.Session;
 import org.apache.juli.logging.Log;
 
 /**
  * Implementation of the {@link org.apache.catalina.Store Store} interface that stores serialized session objects in a
  * database. Sessions that are saved are still subject to being expired based on inactivity.
+ *
+ * @author Bip Thelin
  */
 public class DataSourceStore extends StoreBase {
 
@@ -326,9 +325,11 @@ public class DataSourceStore extends StoreBase {
      * @param expiredOnly flag, whether only keys of expired sessions should be returned
      *
      * @return array containing the list of session IDs
+     *
+     * @exception IOException if an input/output error occurred
      */
-    private String[] keys(boolean expiredOnly) {
-        String[] keys = null;
+    private String[] keys(boolean expiredOnly) throws IOException {
+        String keys[] = null;
         int numberOfTries = 2;
         while (numberOfTries > 0) {
 
@@ -361,7 +362,7 @@ public class DataSourceStore extends StoreBase {
                     }
                 }
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException", e));
                 keys = new String[0];
                 // Close the connection so that it gets reopened next time
             } finally {
@@ -395,7 +396,7 @@ public class DataSourceStore extends StoreBase {
                     numberOfTries = 0;
                 }
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException", e));
             } finally {
                 release(_conn);
             }
@@ -419,7 +420,7 @@ public class DataSourceStore extends StoreBase {
                 return null;
             }
 
-            ClassLoader oldThreadContextCL = context.bind(Globals.IS_SECURITY_ENABLED, null);
+            ClassLoader oldThreadContextCL = context.bind(null);
 
             try (PreparedStatement preparedLoadSql = _conn.prepareStatement(loadSql)) {
                 preparedLoadSql.setString(1, id);
@@ -442,9 +443,9 @@ public class DataSourceStore extends StoreBase {
                     numberOfTries = 0;
                 }
             } catch (SQLException e) {
-                contextLog.error(sm.getString("dataSourceStore.SQLException"), e);
+                contextLog.error(sm.getString("dataSourceStore.SQLException", e));
             } finally {
-                context.unbind(Globals.IS_SECURITY_ENABLED, oldThreadContextCL);
+                context.unbind(oldThreadContextCL);
                 release(_conn);
             }
             numberOfTries--;
@@ -468,7 +469,7 @@ public class DataSourceStore extends StoreBase {
                 // Break out after the finally block
                 numberOfTries = 0;
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException", e));
             } finally {
                 release(_conn);
             }
@@ -516,7 +517,7 @@ public class DataSourceStore extends StoreBase {
                 // Break out after the finally block
                 numberOfTries = 0;
             } catch (SQLException e) {
-                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
+                manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException", e));
             } finally {
                 release(_conn);
             }
@@ -526,6 +527,7 @@ public class DataSourceStore extends StoreBase {
 
     @Override
     public void save(Session session) throws IOException {
+        ByteArrayOutputStream bos = null;
         String saveSql = "INSERT INTO " + sessionTable + " (" + sessionIdCol + ", " + sessionAppCol + ", " +
                 sessionDataCol + ", " + sessionValidCol + ", " + sessionMaxInactiveCol + ", " + sessionLastAccessedCol +
                 ") VALUES (?, ?, ?, ?, ?, ?)";
@@ -539,10 +541,12 @@ public class DataSourceStore extends StoreBase {
                 }
 
                 try {
-                    // Remove session if it exists and insert again.
+                    // If sessions already exist in DB, remove and insert again.
+                    // TODO:
+                    // * Check if ID exists in database and if so use UPDATE.
                     remove(session.getIdInternal(), _conn);
 
-                    ByteArrayOutputStream bos = new ByteArrayOutputStream();
+                    bos = new ByteArrayOutputStream();
                     try (ObjectOutputStream oos = new ObjectOutputStream(new BufferedOutputStream(bos))) {
                         ((StandardSession) session).writeObjectData(oos);
                     }
@@ -562,8 +566,8 @@ public class DataSourceStore extends StoreBase {
                         numberOfTries = 0;
                     }
                 } catch (SQLException e) {
-                    manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException"), e);
-                } catch (IOException ioe) {
+                    manager.getContext().getLogger().error(sm.getString("dataSourceStore.SQLException", e));
+                } catch (IOException e) {
                     // Ignore
                 } finally {
                     release(_conn);
@@ -599,7 +603,7 @@ public class DataSourceStore extends StoreBase {
                 }
             }
         } catch (SQLException ex) {
-            manager.getContext().getLogger().error(sm.getString("dataSourceStore.checkConnectionSQLException"), ex);
+            manager.getContext().getLogger().error(sm.getString("dataSourceStore.checkConnectionSQLException", ex));
         }
 
         return conn;
@@ -615,41 +619,23 @@ public class DataSourceStore extends StoreBase {
     protected Connection open() throws SQLException {
         if (dataSourceName != null && dataSource == null) {
             org.apache.catalina.Context context = getManager().getContext();
+            ClassLoader oldThreadContextCL = null;
             if (getLocalDataSource()) {
-                ClassLoader oldThreadContextCL = context.bind(Globals.IS_SECURITY_ENABLED, null);
-                try {
-                    Context envCtx = (Context) (new InitialContext()).lookup("java:comp/env");
-                    this.dataSource = (DataSource) envCtx.lookup(this.dataSourceName);
-                } catch (NamingException e) {
-                    context.getLogger().error(sm.getString("dataSourceStore.wrongDataSource", this.dataSourceName), e);
-                } finally {
-                    context.unbind(Globals.IS_SECURITY_ENABLED, oldThreadContextCL);
-                }
-            } else {
-                try {
-                    // This should be the normal way to lookup for the global in the global context (no comp/env)
-                    Service service = Container.getService(context);
-                    if (service != null) {
-                        Server server = service.getServer();
-                        if (server != null) {
-                            Context namingContext = server.getGlobalNamingContext();
-                            this.dataSource = (DataSource) namingContext.lookup(dataSourceName);
-                        }
-                    }
-                } catch (NamingException e) {
-                    // Ignore, try another way for compatibility
-                }
-                if (this.dataSource == null) {
-                    try {
-                        Context envCtx = (Context) (new InitialContext()).lookup("java:comp/env");
-                        this.dataSource = (DataSource) envCtx.lookup(this.dataSourceName);
-                    } catch (NamingException e) {
-                        context.getLogger().error(sm.getString("dataSourceStore.wrongDataSource", this.dataSourceName),
-                                e);
-                    }
-                }
+                oldThreadContextCL = context.bind(null);
             }
 
+            Context initCtx;
+            try {
+                initCtx = new InitialContext();
+                Context envCtx = (Context) initCtx.lookup("java:comp/env");
+                this.dataSource = (DataSource) envCtx.lookup(this.dataSourceName);
+            } catch (NamingException e) {
+                context.getLogger().error(sm.getString("dataSourceStore.wrongDataSource", this.dataSourceName), e);
+            } finally {
+                if (getLocalDataSource()) {
+                    context.unbind(oldThreadContextCL);
+                }
+            }
         }
 
         if (dataSource != null) {
@@ -684,7 +670,7 @@ public class DataSourceStore extends StoreBase {
         try {
             dbConnection.close();
         } catch (SQLException e) {
-            manager.getContext().getLogger().error(sm.getString("dataSourceStore.close"), e);
+            manager.getContext().getLogger().error(sm.getString("dataSourceStore.close", e));
         }
     }
 
