@@ -32,7 +32,6 @@ import java.security.cert.Certificate;
 import java.text.Collator;
 import java.util.Arrays;
 import java.util.Locale;
-import java.util.Objects;
 import java.util.jar.JarFile;
 import java.util.jar.Manifest;
 
@@ -40,9 +39,7 @@ import org.apache.catalina.WebResource;
 import org.apache.catalina.WebResourceRoot;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.res.StringManager;
-import org.apache.tomcat.util.security.ConcurrentMessageDigest;
 
 /**
  * This class is designed to wrap a 'raw' WebResource and providing caching for expensive operations. Inexpensive
@@ -76,7 +73,7 @@ public class CachedResource implements WebResource {
     private volatile Boolean cachedExists = null;
     private volatile Boolean cachedIsVirtual = null;
     private volatile Long cachedContentLength = null;
-    private volatile String cachedStrongETag = null;
+    private final Object cachedContentLengthLock = new Object();
 
 
     public CachedResource(Cache cache, StandardRoot root, String path, long ttl, int objectMaxSizeBytes,
@@ -85,7 +82,6 @@ public class CachedResource implements WebResource {
         this.root = root;
         this.webAppPath = path;
         this.ttl = ttl;
-        nextCheck = ttl + System.currentTimeMillis();
         this.objectMaxSizeBytes = objectMaxSizeBytes;
         this.usesClassLoaderResources = usesClassLoaderResources;
     }
@@ -102,12 +98,15 @@ public class CachedResource implements WebResource {
             return false;
         }
 
+        long now = System.currentTimeMillis();
+
         if (webResource == null) {
             synchronized (this) {
                 if (webResource == null) {
                     webResource = root.getResourceInternal(webAppPath, useClassLoaderResources);
                     getLastModified();
                     getContentLength();
+                    nextCheck = ttl + now;
                     // exists() is a relatively expensive check for a file so
                     // use the fact that we know if it exists at this point
                     if (webResource instanceof EmptyResource) {
@@ -120,8 +119,6 @@ public class CachedResource implements WebResource {
             }
         }
 
-        long now = System.currentTimeMillis();
-
         if (now < nextCheck) {
             return true;
         }
@@ -253,20 +250,23 @@ public class CachedResource implements WebResource {
          * 1. It is relatively expensive to calculate, it shouldn't change and this method is called multiple times
          * during cache validation. Caching, therefore, offers a performance benefit.
          *
-         * 2. There is a race condition if concurrent threads are trying to PUT and DELETE the same resource. See BZ
-         * 69527 (https://bz.apache.org/bugzilla/show_bug.cgi?id=69527#c14) for full details. The short version is that
-         * getContentLength() must always return the same value for any one CachedResource instance else the cache size
-         * will be corrupted.
+         * 2. There is a race condition if concurrent threads are trying to PUT and DELETE the same resource. If the
+         * DELETE thread removes the cache entry after the PUT thread has created it but before validation is complete
+         * the DELETE thread sees a content length for a non-existant resource but the PUT thread sees the actual
+         * content length. While that isn't an issue for the individual requests it, does corrupt the cache size
+         * tracking as the size used for the resource is different between when it is added to the cache and when it is
+         * removed. Caching combined with locking ensures that a consistent content length is reported for the resource.
          */
         if (cachedContentLength == null) {
-            if (webResource == null) {
-                synchronized (this) {
+            synchronized (cachedContentLengthLock) {
+                if (cachedContentLength == null) {
                     if (webResource == null) {
-                        webResource = root.getResourceInternal(webAppPath, usesClassLoaderResources);
+                        cachedContentLength = Long.valueOf(0);
+                    } else {
+                        cachedContentLength = Long.valueOf(webResource.getContentLength());
                     }
                 }
             }
-            cachedContentLength = Long.valueOf(webResource.getContentLength());
         }
         return cachedContentLength.longValue();
     }
@@ -291,20 +291,6 @@ public class CachedResource implements WebResource {
         return webResource.getETag();
     }
 
-    @Override
-    public String getStrongETag() {
-        if (cachedStrongETag == null) {
-            byte[] buf = getContent();
-            if (buf != null) {
-                buf = ConcurrentMessageDigest.digestSHA256(buf);
-                cachedStrongETag = "\"" + HexUtils.toHexString(buf) + "\"";
-            } else {
-                cachedStrongETag = webResource.getStrongETag();
-            }
-        }
-        return cachedStrongETag;
-    }
-
     @Override
     public void setMimeType(String mimeType) {
         webResource.setMimeType(mimeType);
@@ -372,6 +358,7 @@ public class CachedResource implements WebResource {
         try {
             CachedResourceURLStreamHandler handler =
                     new CachedResourceURLStreamHandler(resourceURL, root, webAppPath, usesClassLoaderResources);
+            @SuppressWarnings("deprecation")
             URL result = new URL(null, resourceURL.toExternalForm(), handler);
             handler.setCacheURL(result);
             return result;
@@ -422,7 +409,7 @@ public class CachedResource implements WebResource {
         // Longer paths use a noticeable amount of memory so account for this in
         // the cache size. The fixed component of a String instance's memory
         // usage is accounted for in the 500 bytes above.
-        result += getWebappPath().length() * 2L;
+        result += getWebappPath().length() * 2;
         if (getContentLength() <= objectMaxSizeBytes) {
             result += getContentLength();
         }
@@ -592,12 +579,6 @@ public class CachedResource implements WebResource {
         private WebResource getResource() {
             return root.getResource(webAppPath, false, usesClassLoaderResources);
         }
-
-        @Override
-        public String getContentType() {
-            // "content/unknown" is the value used by sun.net.www.URLConnection. It is used here for consistency.
-            return Objects.requireNonNullElse(getResource().getMimeType(), "content/unknown");
-        }
     }
 
 
