@@ -35,7 +35,6 @@ import org.apache.coyote.ActionCode;
 import org.apache.coyote.Adapter;
 import org.apache.coyote.ContinueResponseTiming;
 import org.apache.coyote.ErrorState;
-import org.apache.coyote.NonPipeliningProcessor;
 import org.apache.coyote.Request;
 import org.apache.coyote.RequestGroupInfo;
 import org.apache.coyote.Response;
@@ -53,7 +52,7 @@ import org.apache.tomcat.util.net.SocketEvent;
 import org.apache.tomcat.util.net.SocketWrapperBase;
 import org.apache.tomcat.util.res.StringManager;
 
-class StreamProcessor extends AbstractProcessor implements NonPipeliningProcessor {
+class StreamProcessor extends AbstractProcessor {
 
     private static final Log log = LogFactory.getLog(StreamProcessor.class);
     private static final StringManager sm = StringManager.getManager(StreamProcessor.class);
@@ -147,14 +146,8 @@ class StreamProcessor extends AbstractProcessor implements NonPipeliningProcesso
                     state = SocketState.CLOSED;
                 } finally {
                     if (state == SocketState.CLOSED) {
-                        /*
-                         * Recycle this processor before the stream is recycled as recycling the stream will add the
-                         * request and the response to the pool for re-use (if re-use is enabled) and the request
-                         * statistics updating in StreamProcessor.recycle() needs to happen before the request and
-                         * response are added to the pool to avoid concurrency issues corrupting the statistics.
-                         */
-                        recycle();
                         stream.recycle();
+                        recycle();
                     }
                 }
             } finally {
@@ -262,6 +255,13 @@ class StreamProcessor extends AbstractProcessor implements NonPipeliningProcesso
                 headers.setValue("Server").setString(server);
             }
         }
+
+        // Exclude some HTTP header fields where the value is determined only
+        // while generating the content as per section 9.3.2 of RFC 9110.
+        if (coyoteRequest != null && coyoteRequest.method().equals("HEAD")) {
+            headers.removeHeader("content-length");
+            headers.removeHeader("content-range");
+        }
     }
 
 
@@ -392,23 +392,6 @@ class StreamProcessor extends AbstractProcessor implements NonPipeliningProcesso
     }
 
 
-    @Override
-    protected final boolean isPushSupported() {
-        return stream.isPushSupported();
-    }
-
-
-    @Override
-    protected final void doPush(Request pushTarget) {
-        try {
-            stream.push(pushTarget);
-        } catch (IOException ioe) {
-            setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
-            response.setErrorException(ioe);
-        }
-    }
-
-
     @Override
     protected boolean isTrailerFieldsReady() {
         return stream.isTrailerFieldsReady();
@@ -439,12 +422,6 @@ class StreamProcessor extends AbstractProcessor implements NonPipeliningProcesso
             global.removeRequestProcessor(request.getRequestProcessor());
         }
 
-        /*
-         * Clear the statistics ready for re-use of the request. If we don't clear the statistics, the statistics for
-         * the current request will be included in the statistics for all future requests.
-         */
-        request.getRequestProcessor().recycleStatistcs();
-
         // Clear fields that can be cleared to aid GC and trigger NPEs if this
         // is reused
         setSocketWrapper(null);
